/*
 * WorkspaceChooser.java
 *
 * Created on 3 de Outubro de 2008, 16:23
 */

package org.grview.ui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.grview.project.Project;
import org.grview.util.Log;


/**
 *
 * @author  Gusga
 */
public class WorkspaceChooser extends JFrame {


	private static final long serialVersionUID = 1L;
	private static WorkspaceChooser instance;
	private String workspaceDir;
	private boolean canceled;
	private boolean done;

	private static final String LIST_FILE = "workspace";

	/** Creates new form WorkspaceChooser */
	private WorkspaceChooser() {
		this.setSize(428, 230);
		//place in the middle of screen
		Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(
				(screenDim.width - 428) / 2,
				(screenDim.height - 230) / 2
		);
		this.setResizable(false);
		initComponents();
	}

	public static WorkspaceChooser getInstance() {
		if (instance == null) {
			instance = new WorkspaceChooser();
		}
		return instance;
	}

	public String getWorkspaceDir() {
		return this.workspaceDir;
	}

	public boolean isCanceled() {
		return canceled;
	}

	public boolean isDone() {
		return done;
	}
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		jLabel1 = new javax.swing.JLabel();
		jLabel2 = new javax.swing.JLabel();
		jComboBox1 = new javax.swing.JComboBox();
		btBrowse = new javax.swing.JButton();
		btCancel = new javax.swing.JButton();
		btOk = new javax.swing.JButton();

		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

		jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("projects_screen.png"))); // NOI18N

		jLabel2.setText("Please inform a workspace to continue:");

		btBrowse.setText("Browse");
		btBrowse.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btBrowseActionPerformed(evt);
			}
		});

		btCancel.setText("OK");
		btCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btOkActionPerformed(evt);
			}
		});

		btOk.setText("Cancel");
		btOk.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btCancelActionPerformed(evt);
			}
		});

		jComboBox1.setEditable(true);

		org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(layout.createSequentialGroup()
						.addContainerGap()
						.add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
								.add(jComboBox1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 305, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
								.add(btCancel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 85, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
								.add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
										.add(btOk, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 89, Short.MAX_VALUE)
										.add(btBrowse, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 89, Short.MAX_VALUE))
										.addContainerGap())
										.add(jLabel1)
										.add(layout.createSequentialGroup()
												.addContainerGap()
												.add(jLabel2))
		);
		layout.setVerticalGroup(
				layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(layout.createSequentialGroup()
						.add(jLabel1)
						.addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
						.add(jLabel2)
						.addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
						.add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
								.add(jComboBox1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
								.add(btBrowse))
								.addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
								.add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
										.add(btOk)
										.add(btCancel))
										.addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
		);

		readDirsFromList();
		pack();
	}// </editor-fold>

	private void btBrowseActionPerformed(java.awt.event.ActionEvent evt) {
		JFileChooser jfc = new JFileChooser();
		jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int rVal = jfc.showOpenDialog(this);
		if (rVal == JFileChooser.APPROVE_OPTION) {
			jComboBox1.setSelectedItem(jfc.getSelectedFile().getAbsolutePath());
		}
		if (rVal == JFileChooser.CANCEL_OPTION) {
			jComboBox1.setSelectedItem("");
		}
	}

	private void btOkActionPerformed(java.awt.event.ActionEvent evt) {
		String dir = jComboBox1.getSelectedItem().toString();
		File file = new File(dir);
		boolean newFile = false;
		if (!dir.equals("")) {
			boolean exists = false;
			if (file.exists()) {
				workspaceDir = dir;
			}
			else {
				int option = JOptionPane.showConfirmDialog(this, "This directory does not exist.\nIf you continue this directory will be created and a new project will be created on this directory.\nProceed?", "Directory not found", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if (option == JOptionPane.NO_OPTION) {
					return;
				}
				if (file.mkdir()) {
					newFile = true;
					workspaceDir = dir;
				}
				else {
					JOptionPane.showMessageDialog(this, "Could not find or create this directory in disk!", "Workspace Loader", JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
			try {
				if (!Project.isProject(file)) {
					if (file.listFiles().length > 0) {
						JOptionPane.showMessageDialog(this, "Must be a new, empty, or existing project directory!", "Workspace Loader", JOptionPane.ERROR_MESSAGE);
						return;
					}
					Project.createProject(file, null, null).writeProject();
				}
			}
			catch (Exception e) {
				JOptionPane.showMessageDialog(this, "Could not create a new project!", "Worksapce Loader", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
				if (newFile && !file.delete()) {
					JOptionPane.showMessageDialog(this, "Could not delete the created directory!", "Workspace Loader", JOptionPane.ERROR_MESSAGE);
				}
				return;
			}
			for (int i = 1; i < jComboBox1.getItemCount(); i++) {
				if (jComboBox1.getItemAt(i).toString().equals(dir)) {
					exists = true;
					break;
				}
			}
			if (!exists) { addDirToList(dir); }
		}
		this.setVisible(false);
		done = true;
	}

	private void btCancelActionPerformed(java.awt.event.ActionEvent evt) {
		this.dispose();
		canceled = true;
	}

	private void readDirsFromList() {
		File f = new File(System.getProperty("java.io.tmpdir"),LIST_FILE);
		try {
			if (!f.exists()) {
				f.createNewFile();
			}
			FileReader fr = new FileReader(f);
			BufferedReader br = new BufferedReader(fr);
			String line = "";
			while ((line = br.readLine()) != null) {
				if (!line.equals(""))
					jComboBox1.addItem(line);
			}
		}
		catch (IOException e) {
			Log.log(Log.ERROR, this, "Could not load workspace list!", e);
		}
	}

	private void addDirToList(String filename) {
		File f = new File(System.getProperty("java.io.tmpdir"), LIST_FILE);
		StringBuffer oldText = new StringBuffer();
		try {
			if (!f.exists()) {
				f.createNewFile();
			}
			FileReader fr  = new FileReader(f);
			BufferedReader br = new BufferedReader(fr);
			String line = "";
			while ((line = br.readLine()) != null)
				oldText.append(line + "\n");
			br.close();
			PrintWriter pw = new PrintWriter(f);
			pw.println(oldText.toString());
			pw.print(filename);
			pw.close();
		}
		catch (IOException e) {
			Log.log(Log.ERROR, this, "Could not load workspace list!", e);
		}
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				getInstance().setVisible(true);
			}
		});
	}

	// Variables declaration - do not modify
	private javax.swing.JButton btBrowse;
	private javax.swing.JButton btOk;
	private javax.swing.JButton btCancel;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JComboBox jComboBox1;
	// End of variables declaration

}
